/*
	This file provides a brief demo of some library functions.
	It is not included in your projects.
*/

var/list/colorlist = list()	// a list of all the colors in samplecolors.dm

world/New()
	..()
	// initialize the colorlist
	for(var/T in typesof(/sd_color) - /sd_color)
		var/sd_color/C = new T()
		colorlist[C.name] = C

client/verb
	conversions()
		set desc = "Display the numbers 0 through 16 in decimal, octal, hexidecimal, and binary."

		for(var/n = 0 to 16)
			var/oct = sd_dec2base(n,8,2)
			var/hex = sd_dec2base(n,,2)
			var/binary = sd_dec2base(n,2,8)
			src << "[sd_dec2base(n,10,2)] decimal:\t\
					[oct] oct = [sd_base2dec(oct,8)]\t\
					[hex] hex = [sd_base2dec(hex)]\t\
					[binary] binary = [sd_base2dec(binary,2)]"

	findlast(A as text,B as text)
		set desc = "(haystack, needle) Find the last occurance of needle in the haystack string."

		src << "The last '[B]' in '[A]' is at [sd_findlast(A,B)]."

	replace(A as text,B as text, C as text)
		set desc = "(original_text, old_text, new_text) Replaces all instances of old_text in \
			original_text with new_text."

		src << A
		src << sd_replacetext(A,B,C)

	colors()
		set desc = "Show the name, componant colors, and a demo of all 142 colors in \
			samplecolors.dm."

		for(var/T in colorlist)
			var/sd_color/C = colorlist[T]
			var/nybble = sd_rgb2nybble(C.html)
			var/nybble_rgb = sd_nybble2rgb(nybble)
			src << "<font color=#[C.html]>###</font> <font color=#[nybble_rgb]>###</font> <b>[C.name]</b> R: [C.red]   \
				G: [C.green]   B: [C.blue]   HTML color: [C.html]   Nybble Color: \
				[nybble] - [nybble_rgb]"

	background(color in colorlist)
		set desc = "(color) Set the background color of the text window."

		var/sd_color/C = colorlist[color]
		src << "<STYLE>body {background:#[C.html]}</STYLE>"

	hsl(hue as num, sat as num, lgh as num)
		set desc = "(hue, saturation, light) Convert the HSL colors to RGB and back again."

		var/scale = 240
		src << "hue: [hue]   sat: [sat]   lgh: [lgh]"
		var/RGB = hsl2rgb(hue, sat, lgh, scale)
		src << "RGB: [RGB]   <font color=#[RGB]>***</font>"
		var/HSL = rgb2hsl(RGB, scale)
		src << "HSL: [HSL]"
		hue = sd_base2dec(copytext(HSL,1,3))
		sat = sd_base2dec(copytext(HSL,3,5))
		lgh = sd_base2dec(copytext(HSL,5))
		src << "hue: [hue]   sat: [sat]   lgh: [lgh]"

	hue_sat_chart()
		set desc = "Display a table of hue and saturation values"

		var
			hue
			sat = 240
			lgh = 120
			RGB
		src << "Hue/saturation chart:"
		for(sat = 240 to 0 step -10)
			for(hue = 0 to 240 step 4)
				RGB = hsl2rgb(hue, sat, lgh)
				src << "<font color=#[RGB]>#</font>\..."
			src << ""

	hue_light_chart()
		set desc = "Display a table of hue and light values"

		var
			hue
			sat = 240
			lgh = 120
			RGB
		src << "Hue/light chart:"
		for(lgh = 240 to 0 step -10)
			for(hue = 0 to 240 step 4)
				RGB = hsl2rgb(hue, sat, lgh)
				src << "<font color=#[RGB]>#</font>\..."
			src << ""

	sat_light_chart()
		set desc = "Display a table of saturation and light values"

		var
			hue = 0
			sat = 240
			lgh
			RGB
		src << "Saturation/light chart:"
		for(sat = 240 to 0 step -10)
			for(lgh = 0 to 240 step 4)
				RGB = hsl2rgb(hue, sat, lgh)
				src << "<font color=#[RGB]>#</font>\..."
			src << ""

	profile(N as num)
		src << "Running multiple conversions for profiling."
		for(var/loop = 1 to N)
			src << "loop [loop] of [N]"
			for(var/T in colorlist)
				var/sd_color/C = colorlist[T]
				sd_rgb2nybble(C.html)
		src << "finished"

	encode64(T as text)
		var
			hex = sd_text2hex(T)
			B64 = sd_hex2base64(hex, "*")
			hex2 = sd_base64toHex(B64, "*")
			txt = sd_hex2text(hex2)
		src << "[T] = \nhex: [hex]\nbase 64: [B64]\nhex: [hex2]\n= [txt]"

